import numpy
import math
import scipy.io


def generate_dataset(steps, P0, Q, R):
    x = numpy.zeros((steps + 1,))
    y = numpy.zeros((steps + 1,))
    x[0] = numpy.random.multivariate_normal((0.0,), P0)
    y[0] = 0.05 * x[0] ** 2 + numpy.random.multivariate_normal((0.0,), R)
    for k in range(0, steps):
        x[k + 1] = 0.5 * x[k] + 25.0 * x[k] / (1 + x[k] ** 2) + 8 * math.cos(1.2 * k) + numpy.random.multivariate_normal((0.0,), Q)
        y[k + 1] = 0.05 * x[k + 1] ** 2 + numpy.random.multivariate_normal((0.0,), R)

    return (x, y)



if __name__ == '__main__':

    nbr = 2000
    steps = 100

    P0 = 5.0 * numpy.eye(1)
    Q = 10.0 * numpy.eye(1)
    R = 1.0 * numpy.eye(1)

    x = numpy.empty((nbr, steps + 1))
    y = numpy.empty((nbr, steps + 1))
    for k in xrange(nbr):
        numpy.random.seed(k)
        (xk, yk) = generate_dataset(steps, P0, Q, R)
        x[k, :] = xk
        y[k, :] = yk

    data = scipy.io.savemat("standard_nonlin_dataset.mat", {'x': x, 'y': y})


