'''
Created on Aug 21, 2014

@author: ajn
'''
import numpy
import math
from pyparticleest.simulator import Simulator
from pyparticleest.utils.intrument import Instrumenter as Instrumenter
from pyparticleest.utils.intrument import OpCount as OpCount
import pyparticleest.models.nlg as nlg
from pscomp.common import print_header
from pscomp.common import print_stats
import scipy.io
import matplotlib.pyplot as plt

def generate_dataset(steps, P0, Q, R):
    x = numpy.zeros((steps + 1,))
    y = numpy.zeros((steps + 1,))
    x[0] = numpy.random.normal(0.0, P0)
    y[0] = x[0] + numpy.random.normal(0.0, R)
    for k in range(1, steps + 1):
        x[k] = x[k - 1] + numpy.random.normal(0.0, Q)
        y[k] = x[k] + numpy.random.normal(0.0, R)

    return (x, y)

class Model(nlg.NonlinearGaussianInitialGaussian):
    """ x_{k+1} = x_k + v_k, v_k ~ N(0,Q)
        y_k = x_k + e_k, e_k ~ N(0,R),
        x(0) ~ N(0,P0) """

    def __init__(self, P0, Q, R):
        x0 = numpy.zeros((1, 1))
        super(Model, self).__init__(x0=x0,
                                    Px0=numpy.asarray(P0).reshape((1, 1)),
                                    Q=numpy.asarray(Q).reshape((1, 1)),
                                    R=numpy.asarray(R).reshape((1, 1)))


    def calc_f(self, particles, u, t):
        return particles

    def calc_g(self, particles, t):
        return particles

if __name__ == '__main__':
    steps = 200
    N = 10
    M = 5
    P0 = 1.0
    Q = 1.0
    R = numpy.asarray(((1.0,),))
    numpy.random.seed(0)

    iterations = 500
    print_header()

    model = Model(P0, Q, R)

    Rvec = numpy.asarray([1, 2, 3, 4, 5, 10, 25, 50, 100, 150, 200, 250, 300])
    Rlen = len(Rvec)

    rmse = numpy.empty((Rlen, iterations))
    OpCounts = numpy.empty(Rlen, dtype=object)
    for i in xrange(len(OpCounts)):
        OpCounts[i] = OpCount()
#    plt.ion()
#    plt.figure(1)
#    plt.show()
    for it in xrange(iterations):
        numpy.random.seed(it)
        (x, y) = generate_dataset(steps, P0, Q, R)
        T = len(y)

        instrument = Instrumenter(model)
        sim = Simulator(instrument, None, y)

        sim.simulate(N, M, filter='PF', smoother='ancestor', meas_first=True)

        for r in xrange(1, Rvec[-1] + 1):
            sim.straj.traj = sim.model.pre_mhips_pass(sim.straj)
            sim.straj.traj = sim.straj.perform_mhips_pass(None, reduced=True)
            sim.straj.traj = sim.model.post_smoothing(sim.straj)

            if ((Rvec == r).any()):
                ind = numpy.where(Rvec == r)[0]
                OpCounts[ind] += instrument.oc
                smean = sim.get_smoothed_mean()
                err = smean[:, 0] - x
                rmse[ind, it] = numpy.sqrt(numpy.mean(err ** 2))
#                plt.clf()
#                plt.plot(numpy.arange(T), x, 'r')
#                plt.plot(numpy.arange(T), smean, 'b--')
#                plt.draw()

    for ind in xrange(Rlen):
        print_stats("mhips", N, M, {'R': Rvec[ind]}, numpy.mean(rmse[ind]),
                    numpy.std(rmse[ind]) / math.sqrt(iterations),
                    OpCounts[ind])
