'''
Created on Aug 21, 2014

@author: ajn
'''
import numpy
import math
from pyparticleest.simulator import Simulator
from pyparticleest.utils.intrument import Instrumenter as Instrumenter
from pyparticleest.utils.intrument import OpCount as OpCount
from pscomp.models import Tracking as Tracking
from pscomp.common import print_header2
from pscomp.common import print_stats2
import scipy.io
import matplotlib.pyplot as plt
import scipy.linalg

def generate_data(T, covB, covR):
    x = numpy.zeros((T, 4, 1))
    y = numpy.zeros((T, 2, 1))

    SigmaP = 1.0
    dT = 1.0
    Q = (SigmaP ** 2.0) * numpy.vstack((numpy.hstack(((dT ** 3.0) / 3.0 * numpy.eye(2, 2),
                                                (dT ** 2.0) / 2.0 * numpy.eye(2, 2))),
                                    numpy.hstack(((dT ** 2.0) / 2.0 * numpy.eye(2, 2),
                                                   dT * numpy.eye(2, 2)))))
    Qtri = numpy.triu(scipy.linalg.cho_factor(Q)[0])
    R = numpy.diag((covB, covR))
    Rtri = numpy.triu(scipy.linalg.cho_factor(R)[0])

    Ak = numpy.vstack((numpy.hstack((numpy.eye(2), dT * numpy.eye(2))),
                                numpy.hstack((numpy.zeros((2, 2)), numpy.eye(2)))))

    x[0] = numpy.asarray((-100.0, 50.0, 10.0, 0.0)).reshape((-1, 1))
    y[0] = numpy.vstack((numpy.arctan2(x[0, 1], x[0, 0]),
                         numpy.sqrt(x[0, 0] ** 2 + x[0, 1] ** 2)))
    noise = numpy.random.normal(size=(2, 1))
    y[0] += Rtri.T.dot(noise)
    for i in xrange(1, T):
        noise = numpy.random.normal(size=(4, 1))
        x[i] = Ak.dot(x[i - 1]) + Qtri.T.dot(noise)
        noise = numpy.random.normal(size=(2, 1))
        y[i] = numpy.vstack((numpy.arctan2(x[i, 1], x[i, 0]),
                         numpy.sqrt(x[i, 0] ** 2 + x[i, 1] ** 2)))
        y[i] += Rtri.T.dot(noise)

    return (x, y)

if __name__ == '__main__':

    data = scipy.io.loadmat("data/tracking_data.mat")
    iterations = len(data['data_tracking_720_x'])

    covB = (math.pi / 720.0) ** 2.0
    covR = 0.1

    num = 100

    print_header2()

    model = Tracking(covB=covB, covR=covR)

    Rvec = numpy.asarray([1, 2, 3, 4, 5, 10, 25, 50, 75, 100, 125, 150, 175, 200, 225, 250])
    Rlen = len(Rvec)

    rmse_pos = numpy.empty((Rlen, iterations))
    rmse_vel = numpy.empty((Rlen, iterations))
    OpCounts = numpy.empty(Rlen, dtype=object)
    for i in xrange(len(OpCounts)):
        OpCounts[i] = OpCount()

#    plt.figure()
#    plt.ion()
#    plt.show()
    for it in xrange(iterations):
        x = data['data_tracking_720_x'][it].T
        y = data['data_tracking_720_y'][it].T
        T = len(y)
#        numpy.random.seed(it)
#        T = 50
#        (x, y) = generate_data(T, covB, covR)
#        x = x[:, :, 0]
        instrument = Instrumenter(model)
        sim = Simulator(instrument, None, y)

        sim.simulate(num, 1, filter='pf', smoother='ancestor', meas_first=True)
        ctraj = numpy.copy(sim.straj.traj)
        smean = sim.get_smoothed_mean()
        N = 1.0
        for r in xrange(1, Rvec[-1] + 1):

            sim.simulate(num, 1, filter='cpfas', filter_options={'cond_traj': ctraj},
                         smoother='ancestor', meas_first=True)


            ctraj = numpy.copy(sim.straj.traj)
            N = N + 1.0
            ratio = 1 / N
            smean = (1 - ratio) * smean + ratio * sim.get_smoothed_mean()

            if ((Rvec == r).any()):
                ind = numpy.where(Rvec == r)[0]
                OpCounts[ind] += instrument.oc
                err = smean - x
                rmse_pos[ind, it] = numpy.sqrt(numpy.mean(numpy.sum(err[:, :2] ** 2, axis=1)))
                rmse_vel[ind, it] = numpy.sqrt(numpy.mean(numpy.sum(err[:, 2:] ** 2, axis=1)))

#                if (ind == 0):
#                    plt.figure(1)
#                    plt.plot(x[:, 0], x[:, 1], 'k-')
#                    plt.plot(smean[:, 0], smean[:, 1], 'r--')
#                    plt.figure(2)
#                    plt.plot(x[:, 2], x[:, 3], 'k-')
#                    plt.plot(smean[:, 2], smean[:, 3], 'r--')
#                if (ind == numpy.floor_divide(Rlen, 2)):
#                    plt.figure(1)
#                    plt.plot(smean[:, 0], smean[:, 1], 'g--')
#                    plt.figure(2)
#                    plt.plot(smean[:, 2], smean[:, 3], 'g--')
#                if (ind == Rlen - 1):
#                    plt.figure(1)
#                    plt.plot(smean[:, 0], smean[:, 1], 'b--')
#                    plt.figure(2)
#                    plt.plot(smean[:, 2], smean[:, 3], 'b--')
#                    plt.show()

    for ind in xrange(Rlen):
        print_stats2("pgas", num, 1, {'R': Rvec[ind]},
                     numpy.mean(rmse_pos[ind]), numpy.std(rmse_pos[ind]) / math.sqrt(iterations),
                     numpy.mean(rmse_vel[ind]), numpy.std(rmse_vel[ind]) / math.sqrt(iterations),
                     OpCounts[ind])
