'''
Created on Aug 21, 2014

@author: ajn
'''
import numpy
import math
import scipy.io
from pyparticleest.simulator import Simulator
from pyparticleest.utils.intrument import Instrumenter as Instrumenter
from pscomp.models import Tracking as Tracking
from pscomp.common import parse_setup
from pscomp.common import print_header2
from pscomp.common import print_stats2

if __name__ == '__main__':

    data = scipy.io.loadmat("data/tracking_data.mat")
    iterations = len(data['data_tracking_720_x'])

    covB = (math.pi / 720.0) ** 2.0
    covR = 0.1

    tests = parse_setup()

    print_header2()

    for test in tests:

        N = test[1]
        M = test[2]

        model = Instrumenter(Tracking(covB=covB, covR=covR))
        rmse_pos = numpy.empty((iterations))
        rmse_vel = numpy.empty((iterations))
        enees = numpy.empty((iterations))

        for it in xrange(iterations):

            x = data['data_tracking_720_x'][it].T
            y = data['data_tracking_720_y'][it].T
            T = len(y)

            sim = Simulator(model, None, y)
            sim.simulate(N, M, filter='PF', smoother=test[0], smoother_options=test[3], meas_first=True)

            est_smooth = sim.get_smoothed_mean()

#             plt.figure()
#
#             plt.plot(x[:,0], x[:,1], 'r-', linewidth=2.0, label='True')
#             plt.plot(est_smooth[:,0], est_smooth[:,1], 'b--', linewidth=2.0, label='Smoother mean')
#
#             for j in xrange(M):
#                 plt.plot(sim.straj.traj[:,j,0],sim.straj.traj[:,j,1], 'b--', linewidth=1.0)
#
#             yx = numpy.cos(y[:,0])*y[:,1]
#             yy = numpy.sin(y[:,0])*y[:,1]
#
#             plt.plot(yx,yy, 'rx')
#
#             for i in xrange(0,T,1):
#                 plt.plot(sim.pt.traj[i].pa.part[:,0], sim.pt.traj[i].pa.part[:,1], 'k.', markersize=0.5)
#
#
#             plt.figure()
#
#             plt.plot(x[:,2], x[:,3], 'r--', linewidth=2.0, label='True')
#             plt.plot(est_smooth[:,2], est_smooth[:,3], 'b--', linewidth=2.0, label='Smoother mean')
#
#             plt.show()

            err = est_smooth - x
            rmse_pos[it] = numpy.sqrt(numpy.mean(numpy.sum(err[:, :2] ** 2, axis=1)))
            rmse_vel[it] = numpy.sqrt(numpy.mean(numpy.sum(err[:, 2:] ** 2, axis=1)))

        print_stats2(test[0], N, M, test[3],
                     numpy.mean(rmse_pos), numpy.std(rmse_pos) / math.sqrt(iterations),
                     numpy.mean(rmse_vel), numpy.std(rmse_vel) / math.sqrt(iterations),
                     model.oc)
